@@ -1,4 +1,3 @@
-
 (*
 QCheck: Random testing for OCaml
 copyright (c) 2013-2017, Guillaume Bury, Simon Cruanes, Vincent Hugot, Jan Midtgaard
@@ -7,6 +6,7 @@ all rights reserved.
 
 (** {1 Quickcheck inspired property-based testing} *)
 
+let poly_compare=compare
 open Printf
 
 module RS = Random.State
@@ -75,7 +75,8 @@ module Gen = struct
   let map2 f x y st = f (x st) (y st)
   let map3 f x y z st = f (x st) (y st) (z st)
   let map_keep_input f gen st = let x = gen st in x, f x
-  let (>|=) x f = map f x
+  let (>|=) x f st = f (x st)
+  let (<$>) f x st = f (x st)
 
   let oneof l st = List.nth l (Random.State.int st (List.length l)) st
   let oneofl xs st = List.nth xs (Random.State.int st (List.length xs))
@@ -161,7 +162,17 @@ module Gen = struct
     else fun st -> let r = pint st in r mod (n + 1)
   let int_range a b =
     if b < a then invalid_arg "Gen.int_range";
-    fun st -> a + (int_bound (b-a) st)
+    if a >= 0 || b <= 0 then (* range smaller than max_int *)
+      fun st -> a + (int_bound (b-a) st)
+    else
+      (* range potentially bigger than max_int: we split on 0 and
+         choose the itv wrt to their size ratio *)
+      fun st ->
+      let f_a = float_of_int a in
+      let ratio = (-.f_a) /. (float_of_int b -. f_a) in
+      if Random.float 1. < ratio then - (int_bound a st)
+      else int_bound b st
+
   let (--) = int_range
 
   (* NOTE: we keep this alias to not break code that uses [small_int]
@@ -173,6 +184,8 @@ module Gen = struct
     then small_nat st
     else - (small_nat st)
 
+  let char_range a b = map Char.chr (Char.code a -- Char.code b)
+
   let random_binary_string st length =
     (* 0b011101... *)
     let s = Bytes.create (length + 2) in
@@ -196,6 +209,17 @@ module Gen = struct
   let array gen st = array_size nat gen st
   let array_repeat n g = array_size (return n) g
 
+  let flatten_l l st = List.map (fun f->f st) l
+  let flatten_a a st = Array.map (fun f->f st) a
+  let flatten_opt o st =
+    match o with
+    | None -> None
+    | Some f -> Some (f st)
+  let flatten_res r st =
+    match r with
+    | Ok f -> Ok (f st)
+    | Error e -> Error e
+
   let shuffle_a a st =
     for i = Array.length a-1 downto 1 do
       let j = Random.State.int st (i+1) in
@@ -215,7 +239,7 @@ module Gen = struct
       (float_bound_inclusive 1. st ** (1. /. fl_w), v)
     in
     let samples = List.rev_map sample l in
-    List.sort (fun (w1, _) (w2, _) -> compare w1 w2) samples |> List.rev_map snd
+    List.sort (fun (w1, _) (w2, _) -> poly_compare w1 w2) samples |> List.rev_map snd
 
   let pair g1 g2 st = (g1 st, g2 st)
 
@@ -369,6 +393,24 @@ module Shrink = struct
     if x<0 then yield (x+1);
     ()
 
+  let int32 x yield =
+    let open Int32 in
+    let y = ref x in
+    (* try some divisors *)
+    while !y < -2l || !y > 2l do y := div !y 2l; yield (sub x !y); done; (* fast path *)
+    if x>0l then yield (pred x);
+    if x<0l then yield (succ x);
+    ()
+
+  let int64 x yield =
+    let open Int64 in
+    let y = ref x in
+    (* try some divisors *)
+    while !y < -2L || !y > 2L do y := div !y 2L; yield (sub x !y); done; (* fast path *)
+    if x>0L then yield (pred x);
+    if x<0L then yield (succ x);
+    ()
+
   (* aggressive shrinker for integers,
      get from 0 to x, by dichotomy or just enumerating smaller values *)
   let int_aggressive x yield =
@@ -421,7 +463,7 @@ module Shrink = struct
 
   let list_spine l yield =
     let n = List.length l in
-    let chunk_size = ref (n/2) in
+    let chunk_size = ref ((n+1)/2) in
 
     (* push the [n] first elements of [l] into [q], return the rest of the list *)
     let rec fill_queue n l q = match n,l with
@@ -657,8 +699,12 @@ let small_signed_int = make_int Gen.small_signed_int
 let small_int_corners () = make_int (Gen.nng_corners ())
 let neg_int = make_int Gen.neg_int
 
-let int32 = make_scalar ~print:(fun i -> Int32.to_string i ^ "l") Gen.ui32
-let int64 = make_scalar ~print:(fun i -> Int64.to_string i ^ "L") Gen.ui64
+let int32 =
+  make ~print:(fun i -> Int32.to_string i ^ "l") ~small:small1
+    ~shrink:Shrink.int32 Gen.ui32
+let int64 =
+  make ~print:(fun i -> Int64.to_string i ^ "L") ~small:small1
+    ~shrink:Shrink.int64 Gen.ui64
 
 let char = make_scalar ~print:(sprintf "%C") Gen.char
 let printable_char = make_scalar ~print:(sprintf "%C") Gen.printable
@@ -1167,7 +1213,7 @@ module TestResult = struct
             (* all counter-examples in [l] have same size according to [small],
                so we just compare to the first one, and we enforce
                the invariant *)
-            begin match Pervasives.compare (small instance) (small c_ex'.instance) with
+            begin match poly_compare (small instance) (small c_ex'.instance) with
             | 0 -> res.state <- Failed {instances=c_ex :: l} (* same size: add [c_ex] to [l] *)
             | n when n<0 -> res.state <- Failed {instances=[c_ex]} (* drop [l] *)
             | _ -> () (* drop [c_ex], not small enough *)
@@ -1570,7 +1616,7 @@ module Test = struct
     let median = ref 0 in
     let median_num = ref 0 in (* how many values have we seen yet? once >= !n/2 we set median *)
     (Hashtbl.fold (fun i cnt acc -> (i,cnt)::acc) tbl [])
-    |> List.sort (fun (i,_) (j,_) -> compare i j)
+    |> List.sort (fun (i,_) (j,_) -> poly_compare i j)
     |> List.iter
       (fun (i,cnt) ->
          if !median_num < !num/2 then (
